@@ -1,4 +1,4 @@
-"""Generate langauge specific loaders for a particular SALAD schema."""
+"""Generate language specific loaders for a particular SALAD schema."""
 import sys
 from io import TextIOWrapper
 from typing import (
@@ -15,6 +15,9 @@ from urllib.parse import urlsplit
 
 from . import schema
 from .codegen_base import CodeGenBase
+from .cpp_codegen import CppCodeGen
+from .dlang_codegen import DlangCodeGen
+from .dotnet_codegen import DotNetCodeGen
 from .exceptions import SchemaSaladException
 from .java_codegen import JavaCodeGen
 from .python_codegen import PythonCodeGen
@@ -38,29 +41,51 @@ def codegen(
     parser_info: Optional[str] = None,
 ) -> None:
     """Generate classes with loaders for the given Schema Salad description."""
-
     j = schema.extend_and_specialize(i, loader)
 
-    gen = None  # type: Optional[CodeGenBase]
+    gen: Optional[CodeGenBase] = None
     base = schema_metadata.get("$base", schema_metadata.get("id"))
+    # ``urlsplit`` decides whether to return an encoded result based
+    # on the object type. To ensure the code behaves the same for Py
+    # 3.6+, we enforce that the input value is of type ``str``.
+    if base is None:
+        base = ""
     sp = urlsplit(base)
     pkg = (
         package
         if package
-        else ".".join(
-            list(reversed(sp.netloc.split("."))) + sp.path.strip("/").split("/")
-        )
+        else ".".join(list(reversed(sp.netloc.split("."))) + sp.path.strip("/").split("/"))
     )
     info = parser_info or pkg
-    if lang == "python":
+    if lang == "python" or lang == "cpp" or lang == "dlang":
         if target:
-            dest: Union[TextIOWrapper, TextIO] = open(
-                target, mode="w", encoding="utf-8"
-            )
+            dest: Union[TextIOWrapper, TextIO] = open(target, mode="w", encoding="utf-8")
         else:
             dest = sys.stdout
+        if lang == "cpp":
+            gen = CppCodeGen(
+                base,
+                dest,
+                examples,
+                pkg,
+                copyright,
+            )
+            gen.parse(j)
+            return
+        elif lang == "dlang":
+            gen = DlangCodeGen(
+                base,
+                dest,
+                examples,
+                pkg,
+                copyright,
+                info,
+            )
+            gen.parse(j)
+            return
+        else:
+            gen = PythonCodeGen(dest, copyright=copyright, parser_info=info)
 
-        gen = PythonCodeGen(dest, copyright=copyright, parser_info=info)
     elif lang == "java":
         gen = JavaCodeGen(
             base,
@@ -71,8 +96,10 @@ def codegen(
         )
     elif lang == "typescript":
         gen = TypeScriptCodeGen(base, target=target, package=pkg, examples=examples)
+    elif lang == "dotnet":
+        gen = DotNetCodeGen(base, target=target, package=pkg, examples=examples)
     else:
-        raise SchemaSaladException(f"Unsupported code generation language '{lang}'")
+        raise SchemaSaladException(f"Unsupported code generation language {lang!r}")
 
     gen.prologue()
 
@@ -98,10 +125,7 @@ def codegen(
                 field_name = shortname(field["name"])
                 field_names.append(field_name)
                 tp = field["type"]
-                if (
-                    isinstance(tp, MutableSequence)
-                    and tp[0] == "https://w3id.org/cwl/salad#null"
-                ):
+                if isinstance(tp, MutableSequence) and tp[0] == "https://w3id.org/cwl/salad#null":
                     optional_fields.add(field_name)
 
             idfield = ""
@@ -132,11 +156,12 @@ def codegen(
                     subscope = field.get("subscope")
                     fieldpred = field["name"]
                     optional = bool("https://w3id.org/cwl/salad#null" in field["type"])
-                    uri_loader = gen.uri_loader(
-                        gen.type_loader(field["type"]), True, False, None
-                    )
+                    uri_loader = gen.uri_loader(gen.type_loader(field["type"]), True, False, None)
                     gen.declare_id_field(
-                        fieldpred, uri_loader, field.get("doc"), optional, subscope
+                        fieldpred,
+                        uri_loader,
+                        field.get("doc"),
+                        optional,
                     )
                     break
 
@@ -149,20 +174,17 @@ def codegen(
 
                 if isinstance(jld, MutableMapping):
                     ref_scope = jld.get("refScope")
+                    subscope = jld.get("subscope")
                     if jld.get("typeDSL"):
                         type_loader = gen.typedsl_loader(type_loader, ref_scope)
                     elif jld.get("secondaryFilesDSL"):
                         type_loader = gen.secondaryfilesdsl_loader(type_loader)
-                    elif jld.get("subscope"):
-                        subscope = jld.get("subscope")
                     elif jld.get("_type") == "@id":
                         type_loader = gen.uri_loader(
                             type_loader, jld.get("identity", False), False, ref_scope
                         )
                     elif jld.get("_type") == "@vocab":
-                        type_loader = gen.uri_loader(
-                            type_loader, False, True, ref_scope
-                        )
+                        type_loader = gen.uri_loader(type_loader, False, True, ref_scope)
 
                     map_subject = jld.get("mapSubject")
                     if map_subject:
@@ -179,7 +201,7 @@ def codegen(
                 if jld == "@id":
                     continue
 
-                gen.declare_field(fieldpred, type_loader, field.get("doc"), optional)
+                gen.declare_field(fieldpred, type_loader, field.get("doc"), optional, subscope)
 
             gen.end_class(rec["name"], field_names)
 
