@@ -1,5 +1,6 @@
 """Java code generator for a given schema salad definition."""
 import os
+import re
 import shutil
 import string
 from io import StringIO
@@ -28,6 +29,8 @@ from .schema import shortname
 # may even confuse things a bit so turning these off be default.
 USE_ONE_OR_LIST_OF_TYPES = False
 
+BASIC_JAVA_IDENTIFIER_RE = re.compile(r"[^0-9a-zA-Z]+")
+
 
 def _ensure_directory_and_write(path: Path, contents: str) -> None:
     _safe_makedirs(path.parent)
@@ -142,9 +145,7 @@ class JavaCodeGen(CodeGenBase):
         self.rel_package_dir = Path(rel_package_dir)
         self.main_src_dir = self.target_dir / "src" / "main" / "java" / rel_package_dir
         self.test_src_dir = self.target_dir / "src" / "test" / "java" / rel_package_dir
-        self.test_resources_dir = (
-            self.target_dir / "src" / "test" / "resources" / rel_package_dir
-        )
+        self.test_resources_dir = self.target_dir / "src" / "test" / "resources" / rel_package_dir
 
     def prologue(self) -> None:
         for src_dir in [self.main_src_dir, self.test_src_dir]:
@@ -192,21 +193,15 @@ class JavaCodeGen(CodeGenBase):
             implemented_by = "This interface is implemented by {{@link {}Impl}}<BR>"
             interface_doc_str += implemented_by.format(cls)
         interface_doc_str += doc_to_doc_string(doc)
-        class_doc_str = (
-            f"* Auto-generated class implementation for <I>{classname}</I><BR>"
-        )
+        class_doc_str = f"* Auto-generated class implementation for <I>{classname}</I><BR>"
         class_doc_str += doc_to_doc_string(doc)
         target = self.main_src_dir / f"{cls}.java"
         with open(target, "w") as f:
             _logger.info("Writing file: %s", target)
             if extends:
-                ext = (
-                    "extends "
-                    + ", ".join(self.interface_name(e) for e in extends)
-                    + ", Savable"
-                )
+                ext = "extends " + ", ".join(self.interface_name(e) for e in extends) + ", Saveable"
             else:
-                ext = "extends Savable"
+                ext = "extends Saveable"
             f.write(
                 """// Copyright Common Workflow Language project contributors
 """
@@ -234,7 +229,7 @@ class JavaCodeGen(CodeGenBase):
 
 package {package};
 
-import {package}.utils.Savable;
+import {package}.utils.Saveable;
 
 /**
 {interface_doc_str}
@@ -283,13 +278,13 @@ package {package};
 import {package}.utils.LoaderInstances;
 import {package}.utils.LoadingOptions;
 import {package}.utils.LoadingOptionsBuilder;
-import {package}.utils.SavableImpl;
+import {package}.utils.SaveableImpl;
 import {package}.utils.ValidationException;
 
 /**
 {class_doc_str}
  */
-public class {cls}Impl extends SavableImpl implements {cls} {{
+public class {cls}Impl extends SaveableImpl implements {cls} {{
   private LoadingOptions loadingOptions_ = new LoadingOptionsBuilder().build();
   private java.util.Map<String, Object> extensionFields_ =
       new java.util.HashMap<String, Object>();
@@ -378,9 +373,8 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
 """
             )
 
-    def type_loader(
-        self, type_declaration: Union[List[Any], Dict[str, Any], str]
-    ) -> TypeDef:
+    def type_loader(self, type_declaration: Union[List[Any], Dict[str, Any], str]) -> TypeDef:
+        """Parse the given type declaration and declare its components."""
         if isinstance(type_declaration, MutableSequence):
             sub = [self.type_loader(i) for i in type_declaration]
             if len(sub) < 2:
@@ -418,9 +412,7 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
                     fqclass = f"{self.package}.{single_type.instance_type}"
                     return self.declare_type(
                         TypeDef(
-                            instance_type="{}.utils.OneOrListOf<{}>".format(
-                                self.package, fqclass
-                            ),
+                            instance_type=f"{self.package}.utils.OneOrListOf<{fqclass}>",
                             init="new OneOrListOfLoader<{}>({}, {})".format(
                                 fqclass, single_type.name, array_type.name
                             ),
@@ -458,9 +450,7 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
                         instance_type=instance_type,
                         name=f"array_of_{i.name}",
                         init=f"new ArrayLoader({i.name})",
-                        loader_type="Loader<java.util.List<{}>>".format(
-                            i.instance_type
-                        ),
+                        loader_type=f"Loader<java.util.List<{i.instance_type}>>",
                     )
                 )
             if type_declaration["type"] in ("enum", "https://w3id.org/cwl/salad#enum"):
@@ -470,9 +460,7 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
                 "https://w3id.org/cwl/salad#record",
             ):
                 is_abstract = type_declaration.get("abstract", False)
-                fqclass = "{}.{}".format(
-                    self.package, self.safe_name(type_declaration["name"])
-                )
+                fqclass = "{}.{}".format(self.package, self.safe_name(type_declaration["name"]))
                 return self.declare_type(
                     TypeDef(
                         instance_type=self.safe_name(type_declaration["name"]),
@@ -503,9 +491,7 @@ public class {cls}Impl extends SavableImpl implements {cls} {{
         for sym in symbols:
             self.add_vocab(shortname(sym), sym)
         clazz = self.safe_name(type_declaration["name"])
-        symbols_decl = 'new String[] {{"{}"}}'.format(
-            '", "'.join(sym for sym in symbols)
-        )
+        symbols_decl = 'new String[] {{"{}"}}'.format('", "'.join(sym for sym in symbols))
         enum_path = self.main_src_dir / f"{clazz}.java"
         with open(enum_path, "w") as f:
             _logger.info("Writing file: %s", enum_path)
@@ -546,11 +532,7 @@ public enum {clazz} {{
             for i, sym in enumerate(symbols):
                 suffix = "," if i < (len(symbols) - 1) else ";"
                 const = self.safe_name(sym).replace("-", "_").replace(".", "_").upper()
-                f.write(
-                    """  {const}("{val}"){suffix}\n""".format(
-                        const=const, val=sym, suffix=suffix
-                    )
-                )
+                f.write(f"""  {const}("{sym}"){suffix}\n""")  # noqa: B907
             f.write(
                 """
   private static String[] symbols = {symbols_decl};
@@ -588,6 +570,7 @@ public enum {clazz} {{
         fieldtype: TypeDef,
         doc: Optional[str],
         optional: bool,
+        subscope: str,
     ) -> None:
         fieldname = name
         property_name = self.property_name(fieldname)
@@ -689,12 +672,11 @@ public enum {clazz} {{
         fieldtype: TypeDef,
         doc: str,
         optional: bool,
-        subscope: Optional[str],
     ) -> None:
         if self.current_class_is_abstract:
             return
 
-        self.declare_field(name, fieldtype, doc, True)
+        self.declare_field(name, fieldtype, doc, True, "")
         if optional:
             set_uri = """
     Boolean __original_is_null = {safename} == null;
@@ -722,8 +704,6 @@ public enum {clazz} {{
     }}
     __baseUri = (String) {safename};
 """
-        if subscope is not None:
-            name = name + subscope
 
         self.current_loader.write(
             set_uri.format(safename=self.safe_name(name), fieldname=shortname(name))
@@ -740,9 +720,7 @@ public enum {clazz} {{
         return self.declare_type(
             TypeDef(
                 instance_type=instance_type,  # ?
-                name="uri_{}_{}_{}_{}".format(
-                    inner.name, scoped_id, vocab_term, ref_scope
-                ),
+                name=f"uri_{inner.name}_{scoped_id}_{vocab_term}_{ref_scope}",
                 init="new UriLoader({}, {}, {}, {})".format(
                     inner.name,
                     self.to_java(scoped_id),
@@ -811,9 +789,9 @@ public enum {clazz} {{
         )
 
         def template_from_resource(resource: str) -> string.Template:
-            template_str = pkg_resources.resource_string(
-                __name__, f"java/{resource}"
-            ).decode("utf-8")
+            template_str = pkg_resources.resource_string(__name__, f"java/{resource}").decode(
+                "utf-8"
+            )
             template = string.Template(template_str)
             return template
 
@@ -838,8 +816,8 @@ public enum {clazz} {{
         vocab = ""
         rvocab = ""
         for k in sorted(self.vocab.keys()):
-            vocab += f"""    vocab.put("{k}", "{self.vocab[k]}");\n"""
-            rvocab += f"""    rvocab.put("{self.vocab[k]}", "{k}");\n"""
+            vocab += f"""    vocab.put("{k}", "{self.vocab[k]}");\n"""  # noqa: B907
+            rvocab += f"""    rvocab.put("{self.vocab[k]}", "{k}");\n"""  # noqa: B907
 
         loader_instances = ""
         for _, collected_type in self.collected_types.items():
@@ -857,6 +835,7 @@ public enum {clazz} {{
             for example_name in os.listdir(self.examples):
                 if example_name.startswith("valid"):
                     basename = os.path.basename(example_name).rsplit(".", 1)[0]
+                    basename = re.sub(BASIC_JAVA_IDENTIFIER_RE, "_", basename)
                     example_tests += """
   @org.junit.Test
   public void test{basename}ByString() throws Exception {{
@@ -882,7 +861,7 @@ public enum {clazz} {{
     doc = (java.util.Map<String, Object>) YamlUtils.mapFromString(yaml);
     RootLoader.loadDocument(doc, url.toString());
   }}""".format(
-                        basename=basename.replace("-", "_").replace(".", "_"),
+                        basename=basename,
                         example_name=example_name,
                     )
 
@@ -900,7 +879,7 @@ public enum {clazz} {{
             "main_utils": self.main_src_dir,
             "test_utils": self.test_src_dir,
         }
-        for (util_src, util_target) in util_src_dirs.items():
+        for util_src, util_target in util_src_dirs.items():
             for util in pkg_resources.resource_listdir(__name__, f"java/{util_src}"):
                 src_path = util_target / "utils" / util
                 src_template = template_from_resource(os.path.join(util_src, util))
