@@ -76,9 +76,7 @@ def arg_parser() -> argparse.ArgumentParser:
         "--print-pre", action="store_true", help="Print document after preprocessing"
     )
     exgroup.add_argument("--print-index", action="store_true", help="Print node index")
-    exgroup.add_argument(
-        "--print-metadata", action="store_true", help="Print document metadata"
-    )
+    exgroup.add_argument("--print-metadata", action="store_true", help="Print document metadata")
     exgroup.add_argument(
         "--print-inheritance-dot",
         action="store_true",
@@ -95,14 +93,14 @@ def arg_parser() -> argparse.ArgumentParser:
         type=str,
         metavar="language",
         help="Generate classes in target language, currently supported: "
-        "python, java, typescript",
+        "python, java, typescript, dotnet, cpp, dlang",
     )
 
     parser.add_argument(
         "--codegen-target",
         type=str,
         default=None,
-        help="Defaults to sys.stdout for python and ./ for Java",
+        help="Defaults to sys.stdout for Python/C++/Dlang and ./ for " "Java/TypeScript/.Net",
     )
 
     parser.add_argument(
@@ -110,7 +108,7 @@ def arg_parser() -> argparse.ArgumentParser:
         type=str,
         metavar="directory",
         default=None,
-        help="Directory of example documents for test case generation (Java only).",
+        help="Directory of example documents for test case generation (Java/TypeScript/.Net/Dlang only).",
     )
 
     parser.add_argument(
@@ -119,7 +117,7 @@ def arg_parser() -> argparse.ArgumentParser:
         metavar="dotted.package",
         default=None,
         help="Optional override of the package name which is other derived "
-        "from the base URL (Java only).",
+        "from the base URL (Java/TypeScript/.Net/Dlang only).",
     ),
 
     parser.add_argument(
@@ -135,7 +133,7 @@ def arg_parser() -> argparse.ArgumentParser:
         metavar="parser_info",
         type=str,
         default=None,
-        help="Optional parser name which is accessible via resulted parser API (Python only)",
+        help="Optional parser name which is accessible via resulted parser API (Python and Dlang only)",
     )
 
     exgroup.add_argument(
@@ -165,15 +163,11 @@ def arg_parser() -> argparse.ArgumentParser:
     )
 
     exgroup_volume = parser.add_mutually_exclusive_group()
-    exgroup_volume.add_argument(
-        "--verbose", action="store_true", help="Default logging"
-    )
+    exgroup_volume.add_argument("--verbose", action="store_true", help="Default logging")
     exgroup_volume.add_argument(
         "--quiet", action="store_true", help="Only print warnings and errors."
     )
-    exgroup_volume.add_argument(
-        "--debug", action="store_true", help="Print even more logging"
-    )
+    exgroup_volume.add_argument("--debug", action="store_true", help="Print even more logging")
 
     parser.add_argument(
         "--only",
@@ -185,9 +179,7 @@ def arg_parser() -> argparse.ArgumentParser:
         action="append",
         help="Use with --print-doc, override default link for type",
     )
-    parser.add_argument(
-        "--brand", help="Use with --print-doc, set the 'brand' text in nav bar"
-    )
+    parser.add_argument("--brand", help="Use with --print-doc, set the 'brand' text in nav bar")
     parser.add_argument(
         "--brandlink",
         help="Use with --print-doc, set the link for 'brand' in nav bar",
@@ -210,9 +202,7 @@ def arg_parser() -> argparse.ArgumentParser:
 
     parser.add_argument("schema", type=str, nargs="?", default=None)
     parser.add_argument("document", type=str, nargs="?", default=None)
-    parser.add_argument(
-        "--version", "-v", action="store_true", help="Print version", default=None
-    )
+    parser.add_argument("--version", "-v", action="store_true", help="Print version", default=None)
     return parser
 
 
@@ -245,16 +235,12 @@ def main(argsl: Optional[List[str]] = None) -> int:
     # Load schema document and resolve refs
 
     schema_uri = args.schema
-    if not (
-        urlparse(schema_uri)[0] and urlparse(schema_uri)[0] in ["http", "https", "file"]
-    ):
+    if not (urlparse(schema_uri)[0] and urlparse(schema_uri)[0] in ["http", "https", "file"]):
         schema_uri = file_uri(os.path.abspath(schema_uri))
     schema_raw_doc = metaschema_loader.fetch(schema_uri)
 
     try:
-        schema_doc, schema_metadata = metaschema_loader.resolve_all(
-            schema_raw_doc, schema_uri
-        )
+        schema_doc, schema_metadata = metaschema_loader.resolve_all(schema_raw_doc, schema_uri)
     except ValidationException as e:
         _logger.error(
             "Schema `%s` failed link checking:\n%s",
@@ -265,7 +251,7 @@ def main(argsl: Optional[List[str]] = None) -> int:
         _logger.debug("Index is %s", list(metaschema_loader.idx.keys()))
         _logger.debug("Vocabulary is %s", list(metaschema_loader.vocab.keys()))
         return 1
-    except (RuntimeError) as e:
+    except RuntimeError as e:
         _logger.error(
             "Schema `%s` read error:\n%s",
             args.schema,
@@ -294,16 +280,12 @@ def main(argsl: Optional[List[str]] = None) -> int:
         return 0
 
     if not args.document and args.print_index:
-        json_dump(
-            list(metaschema_loader.idx.keys()), fp=sys.stdout, indent=4, default=str
-        )
+        json_dump(list(metaschema_loader.idx.keys()), fp=sys.stdout, indent=4, default=str)
         return 0
 
     # Validate the schema document against the metaschema
     try:
-        schema.validate_doc(
-            metaschema_names, schema_doc, metaschema_loader, args.strict
-        )
+        schema.validate_doc(metaschema_names, schema_doc, metaschema_loader, args.strict)
     except ValidationException as e:
         _logger.error("While validating schema `%s`:\n%s", args.schema, str(e))
         return 1
@@ -315,9 +297,7 @@ def main(argsl: Optional[List[str]] = None) -> int:
     if isinstance(schema_doc, CommentedSeq):
         (schema_ctx, rdfs) = jsonld_context.salad_to_jsonld_context(schema_doc, metactx)
     else:
-        raise ValidationException(
-            f"Expected a CommentedSeq, got {type(schema_doc)}: {schema_doc}."
-        )
+        raise ValidationException(f"Expected a CommentedSeq, got {type(schema_doc)}: {schema_doc}.")
 
     # Create the loader that will be used to load the target document.
     document_loader = Loader(schema_ctx, skip_schemas=args.skip_schemas)
@@ -353,7 +333,7 @@ def main(argsl: Optional[List[str]] = None) -> int:
                 json_dump(avsc_obj, fp=sys.stdout, indent=4, default=str)
             return 1
     else:
-        _logger.error("Schema `%s` must be a list.", args.schema)
+        _logger.error("Schema `%s` must be a list.", args.schema)  # type: ignore[unreachable]
         return 1
 
     # Optionally print Avro-compatible schema from schema
@@ -411,9 +391,7 @@ def main(argsl: Optional[List[str]] = None) -> int:
         return 0
 
     if args.print_index:
-        json_dump(
-            list(document_loader.idx.keys()), fp=sys.stdout, indent=4, default=str
-        )
+        json_dump(list(document_loader.idx.keys()), fp=sys.stdout, indent=4, default=str)
         return 0
 
     # Validate the user document against the schema
@@ -443,7 +421,7 @@ def main(argsl: Optional[List[str]] = None) -> int:
         json_dump(doc_metadata, fp=sys.stdout, indent=4, default=str)
         return 0
 
-    print(f"Document `{args.document}` is valid")
+    _logger.info(f"Document `{args.document}` is valid")
 
     return 0
 
