@@ -66,8 +66,8 @@ def fmt(text: str, indent: int) -> str:
     return textwrap.indent(
         black.format_str(
             text,
-            mode=black.Mode(
-                target_versions={black.TargetVersion.PY36}, line_length=88 - indent
+            mode=black.mode.Mode(
+                target_versions={black.mode.TargetVersion.PY36}, line_length=88 - indent
             ),
         ),
         " " * indent,
@@ -97,10 +97,12 @@ class PythonCodeGen(CodeGenBase):
         avn = schema.avro_field_name(name)
         if avn.startswith("anon."):
             avn = avn[5:]
-        if avn in ("class", "in"):
+        elif avn[0].isdigit():
+            avn = f"_{avn}"
+        elif avn in ("class", "in"):
             # reserved words
-            avn = avn + "_"
-        return avn
+            avn = f"{avn}_"
+        return avn.replace(".", "_")
 
     def prologue(self) -> None:
         """Trigger to generate the prolouge code."""
@@ -132,11 +134,11 @@ class PythonCodeGen(CodeGenBase):
     return "{self.parser_info}"
 
 
-"""
+"""  # noqa: B907
         )
 
-        for primative in prims.values():
-            self.declare_type(primative)
+        for primitive in prims.values():
+            self.declare_type(primitive)
 
     def begin_class(
         self,  # pylint: disable=too-many-arguments
@@ -153,7 +155,7 @@ class PythonCodeGen(CodeGenBase):
         if extends:
             ext = ", ".join(self.safe_name(e) for e in extends)
         else:
-            ext = "Savable"
+            ext = "Saveable"
 
         self.out.write(fmt(f"class {classname}({ext}):\n    pass", 0)[:-9])
         # make a valid class for Black, but then trim off the "pass"
@@ -173,11 +175,7 @@ class PythonCodeGen(CodeGenBase):
 
         safe_inits: List[str] = ["        self,"]
         safe_inits.extend(
-            [
-                f"        {self.safe_name(f)}: Any,"
-                for f in required_field_names
-                if f != "class"
-            ]
+            [f"        {self.safe_name(f)}: Any," for f in required_field_names if f != "class"]
         )
         safe_inits.extend(
             [
@@ -215,9 +213,42 @@ class PythonCodeGen(CodeGenBase):
 """.format(
                     self.safe_name(name)
                 )
+        field_eqs = []
+        field_hashes = []
+        for name in field_names:
+            field_eqs.append("self.{0} == other.{0}".format(self.safe_name(name)))
+            field_hashes.append(f"self.{self.safe_name(name)}")
+        field_eq = " and\n                    ".join(field_eqs)
+        field_hash = ",\n            ".join(field_hashes)
+        self.out.write(field_inits)
         self.out.write(
-            field_inits
-            + f"""
+            "\n"
+            + fmt(
+                f"""def __eq__(
+    self,
+    other: Any
+) -> bool:
+    if isinstance(other, {classname}):
+        return bool({field_eq})
+    return False
+""",
+                4,
+            )
+        )
+        self.out.write(
+            "\n"
+            + fmt(
+                f"""def __hash__(self) -> int:
+    return hash((
+        {field_hash}
+    ))
+""",
+                4,
+            )
+        )
+
+        self.out.write(
+            f"""
     @classmethod
     def fromDoc(
         cls,
@@ -231,7 +262,7 @@ class PythonCodeGen(CodeGenBase):
             _doc.lc.data = doc.lc.data
             _doc.lc.filename = doc.lc.filename
         _errors__ = []
-"""
+"""  # noqa: B907
         )
 
         self.idfield = idfield
@@ -242,8 +273,13 @@ class PythonCodeGen(CodeGenBase):
         self, top: bool = False, base_url: str = "", relative_uris: bool = True
     ) -> Dict[str, Any]:
         r: Dict[str, Any] = {}
-        for ef in self.extension_fields:
-            r[prefix_url(ef, self.loadingOptions.vocab)] = self.extension_fields[ef]
+
+        if relative_uris:
+            for ef in self.extension_fields:
+                r[prefix_url(ef, self.loadingOptions.vocab)] = self.extension_fields[ef]
+        else:
+            for ef in self.extension_fields:
+                r[ef] = self.extension_fields[ef]
 """
         )
 
@@ -317,7 +353,10 @@ if _errors__:
         self.serializer.write("        return r\n\n")
 
         self.serializer.write(
-            fmt(f"""attrs = frozenset(["{'", "'.join(field_names)}"])\n""", 4)
+            fmt(
+                f"""attrs = frozenset(["{'", "'.join(field_names)}"])\n""",  # noqa: B907
+                4,
+            )
         )
 
         safe_init_fields = [
@@ -326,26 +365,28 @@ if _errors__:
 
         safe_inits = [f + "=" + f for f in safe_init_fields]
 
-        safe_inits.extend(
-            ["extension_fields=extension_fields", "loadingOptions=loadingOptions"]
-        )
+        safe_inits.extend(["extension_fields=extension_fields", "loadingOptions=loadingOptions"])
 
         self.out.write(
-            "        return cls(\n            "
+            "        _constructed = cls(\n            "
             + ",\n            ".join(safe_inits)
             + ",\n        )\n"
         )
+        if self.idfield:
+            self.out.write(
+                f"        loadingOptions.idx[{self.safe_name(self.idfield)}] "
+                "= (_constructed, loadingOptions)\n"
+            )
+
+        self.out.write("        return _constructed\n")
 
         self.out.write(str(self.serializer.getvalue()))
 
         self.out.write("\n\n")
 
-    def type_loader(
-        self, type_declaration: Union[List[Any], Dict[str, Any], str]
-    ) -> TypeDef:
+    def type_loader(self, type_declaration: Union[List[Any], Dict[str, Any], str]) -> TypeDef:
         """Parse the given type declaration and declare its components."""
         if isinstance(type_declaration, MutableSequence):
-
             sub_names: List[str] = list(
                 dict.fromkeys([self.type_loader(i).name for i in type_declaration])
             )
@@ -361,20 +402,28 @@ if _errors__:
                 "https://w3id.org/cwl/salad#array",
             ):
                 i = self.type_loader(type_declaration["items"])
-                return self.declare_type(
-                    TypeDef(f"array_of_{i.name}", f"_ArrayLoader({i.name})")
-                )
+                return self.declare_type(TypeDef(f"array_of_{i.name}", f"_ArrayLoader({i.name})"))
             if type_declaration["type"] in ("enum", "https://w3id.org/cwl/salad#enum"):
                 for sym in type_declaration["symbols"]:
                     self.add_vocab(shortname(sym), sym)
+                if "doc" in type_declaration:
+                    doc = type_declaration["doc"]
+                    if isinstance(doc, MutableSequence):
+                        formated_doc = "\n".join(doc)
+                    else:
+                        formated_doc = doc.strip()
+                    docstring = f'\n"""\n{formated_doc}\n"""'
+                else:
+                    docstring = ""
                 return self.declare_type(
                     TypeDef(
                         self.safe_name(type_declaration["name"]) + "Loader",
-                        '_EnumLoader(("{}",))'.format(
+                        '_EnumLoader(("{}",), "{}"){}'.format(
                             '", "'.join(
-                                self.safe_name(sym)
-                                for sym in type_declaration["symbols"]
-                            )
+                                schema.avro_field_name(sym) for sym in type_declaration["symbols"]
+                            ),
+                            self.safe_name(type_declaration["name"]),
+                            docstring,
                         ),
                     )
                 )
@@ -412,12 +461,11 @@ if _errors__:
         fieldtype: TypeDef,
         doc: str,
         optional: bool,
-        subscope: Optional[str],
     ) -> None:
         if self.current_class_is_abstract:
             return
 
-        self.declare_field(name, fieldtype, doc, True)
+        self.declare_field(name, fieldtype, doc, True, "")
 
         if optional:
             opt = """{safename} = "_:" + str(_uuid__.uuid4())""".format(
@@ -428,9 +476,6 @@ if _errors__:
                 fieldname=shortname(name)
             )
 
-        if subscope is not None:
-            name = name + subscope
-
         self.out.write(
             """
         __original_{safename}_is_none = {safename} is None
@@ -447,9 +492,13 @@ if _errors__:
         )
 
     def declare_field(
-        self, name: str, fieldtype: TypeDef, doc: Optional[str], optional: bool
+        self,
+        name: str,
+        fieldtype: TypeDef,
+        doc: Optional[str],
+        optional: bool,
+        subscope: str,
     ) -> None:
-
         if self.current_class_is_abstract:
             return
 
@@ -457,16 +506,29 @@ if _errors__:
             return
 
         if optional:
-            self.out.write(f"""        if "{shortname(name)}" in _doc:\n""")
+            self.out.write(f"""        if "{shortname(name)}" in _doc:\n""")  # noqa: B907
             spc = "    "
         else:
             spc = ""
+
+        if subscope:
+            self.out.write(
+                """
+{spc}        subscope_baseuri = expand_url('{subscope}', baseuri, loadingOptions, True)
+""".format(
+                    subscope=subscope, spc=spc
+                )
+            )
+            baseurivar = "subscope_baseuri"
+        else:
+            baseurivar = "baseuri"
+
         self.out.write(
             """{spc}        try:
 {spc}            {safename} = load_field(
 {spc}                _doc.get("{fieldname}"),
 {spc}                {fieldtype},
-{spc}                baseuri,
+{spc}                {baseurivar},
 {spc}                loadingOptions,
 {spc}            )
 {spc}        except ValidationException as e:
@@ -481,6 +543,7 @@ if _errors__:
                 safename=self.safe_name(name),
                 fieldname=shortname(name),
                 fieldtype=fieldtype.name,
+                baseurivar=baseurivar,
                 spc=spc,
             )
         )
@@ -504,8 +567,7 @@ if _errors__:
                     """
 if self.{safename} is not None:
     u = save_relative_uri(self.{safename}, {baseurl}, {scoped_id}, {ref_scope}, relative_uris)
-    if u:
-        r["{fieldname}"] = u
+    r["{fieldname}"] = u
 """.format(
                         safename=self.safe_name(name),
                         fieldname=shortname(name).strip(),
@@ -544,9 +606,7 @@ if self.{safename} is not None:
         return self.declare_type(
             TypeDef(
                 f"uri_{inner.name}_{scoped_id}_{vocab_term}_{ref_scope}",
-                "_URILoader({}, {}, {}, {})".format(
-                    inner.name, scoped_id, vocab_term, ref_scope
-                ),
+                f"_URILoader({inner.name}, {scoped_id}, {vocab_term}, {ref_scope})",
                 is_uri=True,
                 scoped_id=scoped_id,
                 ref_scope=ref_scope,
@@ -560,9 +620,7 @@ if self.{safename} is not None:
         return self.declare_type(
             TypeDef(
                 f"idmap_{self.safe_name(field)}_{inner.name}",
-                "_IdMapLoader({}, '{}', '{}')".format(
-                    inner.name, map_subject, map_predicate
-                ),
+                f"_IdMapLoader({inner.name}, '{map_subject}', '{map_predicate}')",  # noqa: B907
             )
         )
 
@@ -588,19 +646,17 @@ if self.{safename} is not None:
         """Trigger to generate the epilouge code."""
         self.out.write("_vocab = {\n")
         for k in sorted(self.vocab.keys()):
-            self.out.write(f'    "{k}": "{self.vocab[k]}",\n')
+            self.out.write(f'    "{k}": "{self.vocab[k]}",\n')  # noqa: B907
         self.out.write("}\n")
 
         self.out.write("_rvocab = {\n")
         for k in sorted(self.vocab.keys()):
-            self.out.write(f'    "{self.vocab[k]}": "{k}",\n')
+            self.out.write(f'    "{self.vocab[k]}": "{k}",\n')  # noqa: B907
         self.out.write("}\n\n")
 
         for _, collected_type in self.collected_types.items():
             if not collected_type.abstract:
-                self.out.write(
-                    fmt(f"{collected_type.name} = {collected_type.init}\n", 0)
-                )
+                self.out.write(fmt(f"{collected_type.name} = {collected_type.init}\n", 0))
         self.out.write("\n")
 
         self.out.write(
@@ -614,11 +670,31 @@ def load_document(
         baseuri = file_uri(os.getcwd()) + "/"
     if loadingOptions is None:
         loadingOptions = LoadingOptions()
+    result, metadata = _document_load(
+        %(name)s,
+        doc,
+        baseuri,
+        loadingOptions,
+    )
+    return result
+
+
+def load_document_with_metadata(
+    doc: Any,
+    baseuri: Optional[str] = None,
+    loadingOptions: Optional[LoadingOptions] = None,
+    addl_metadata_fields: Optional[MutableSequence[str]] = None,
+) -> Any:
+    if baseuri is None:
+        baseuri = file_uri(os.getcwd()) + "/"
+    if loadingOptions is None:
+        loadingOptions = LoadingOptions(fileuri=baseuri)
     return _document_load(
         %(name)s,
         doc,
         baseuri,
         loadingOptions,
+        addl_metadata_fields=addl_metadata_fields,
     )
 
 
@@ -633,14 +709,14 @@ def load_document_by_string(
 
     if loadingOptions is None:
         loadingOptions = LoadingOptions(fileuri=uri)
-    loadingOptions.idx[uri] = result
 
-    return _document_load(
+    result, metadata = _document_load(
         %(name)s,
         result,
         uri,
         loadingOptions,
     )
+    return result
 
 
 def load_document_by_yaml(
@@ -660,14 +736,14 @@ def load_document_by_yaml(
 
     if loadingOptions is None:
         loadingOptions = LoadingOptions(fileuri=uri)
-    loadingOptions.idx[uri] = yaml
 
-    return _document_load(
+    result, metadata = _document_load(
         %(name)s,
         yaml,
         uri,
         loadingOptions,
     )
+    return result
 """
             % dict(name=root_loader.name)
         )
