@@ -1,10 +1,12 @@
 """Template code used by python_codegen.py."""
 import copy
+import logging
 import os
 import pathlib
 import re
 import tempfile
 import uuid as _uuid__  # pylint: disable=unused-import # noqa: F401
+import xml.sax  # nosec
 from abc import ABC, abstractmethod
 from io import StringIO
 from typing import (
@@ -18,49 +20,106 @@ from typing import (
     Tuple,
     Type,
     Union,
+    cast,
 )
-from urllib.parse import quote, urlsplit, urlunsplit
+from urllib.parse import quote, urldefrag, urlparse, urlsplit, urlunsplit
 from urllib.request import pathname2url
 
+from rdflib import Graph
+from rdflib.plugins.parsers.notation3 import BadSyntax
 from ruamel.yaml.comments import CommentedMap
 
 from schema_salad.exceptions import SchemaSaladException, ValidationException
-from schema_salad.fetcher import DefaultFetcher, Fetcher
+from schema_salad.fetcher import DefaultFetcher, Fetcher, MemoryCachingFetcher
 from schema_salad.sourceline import SourceLine, add_lc_filename
-from schema_salad.utils import yaml_no_ts  # requires schema-salad v8.2+
+from schema_salad.utils import CacheType, yaml_no_ts  # requires schema-salad v8.2+
 
 _vocab: Dict[str, str] = {}
 _rvocab: Dict[str, str] = {}
 
+_logger = logging.getLogger("salad")
+
+
+IdxType = MutableMapping[str, Tuple[Any, "LoadingOptions"]]
+
 
 class LoadingOptions:
+    idx: IdxType
+    fileuri: Optional[str]
+    baseuri: str
+    namespaces: MutableMapping[str, str]
+    schemas: MutableSequence[str]
+    original_doc: Optional[Any]
+    addl_metadata: MutableMapping[str, Any]
+    fetcher: Fetcher
+    vocab: Dict[str, str]
+    rvocab: Dict[str, str]
+    cache: CacheType
+    imports: List[str]
+    includes: List[str]
+
     def __init__(
         self,
         fetcher: Optional[Fetcher] = None,
         namespaces: Optional[Dict[str, str]] = None,
-        schemas: Optional[Dict[str, str]] = None,
+        schemas: Optional[List[str]] = None,
         fileuri: Optional[str] = None,
         copyfrom: Optional["LoadingOptions"] = None,
         original_doc: Optional[Any] = None,
+        addl_metadata: Optional[Dict[str, str]] = None,
+        baseuri: Optional[str] = None,
+        idx: Optional[IdxType] = None,
+        imports: Optional[List[str]] = None,
+        includes: Optional[List[str]] = None,
     ) -> None:
         """Create a LoadingOptions object."""
-        self.idx: Dict[str, Dict[str, Any]] = {}
-        self.fileuri: Optional[str] = fileuri
-        self.namespaces = namespaces
-        self.schemas = schemas
         self.original_doc = original_doc
-        if copyfrom is not None:
-            self.idx = copyfrom.idx
-            if fetcher is None:
-                fetcher = copyfrom.fetcher
-            if fileuri is None:
-                self.fileuri = copyfrom.fileuri
-            if namespaces is None:
-                self.namespaces = copyfrom.namespaces
-            if schemas is None:
-                self.schemas = copyfrom.schemas
 
-        if fetcher is None:
+        if idx is not None:
+            self.idx = idx
+        else:
+            self.idx = copyfrom.idx if copyfrom is not None else {}
+
+        if fileuri is not None:
+            self.fileuri = fileuri
+        else:
+            self.fileuri = copyfrom.fileuri if copyfrom is not None else None
+
+        if baseuri is not None:
+            self.baseuri = baseuri
+        else:
+            self.baseuri = copyfrom.baseuri if copyfrom is not None else ""
+
+        if namespaces is not None:
+            self.namespaces = namespaces
+        else:
+            self.namespaces = copyfrom.namespaces if copyfrom is not None else {}
+
+        if schemas is not None:
+            self.schemas = schemas
+        else:
+            self.schemas = copyfrom.schemas if copyfrom is not None else []
+
+        if addl_metadata is not None:
+            self.addl_metadata = addl_metadata
+        else:
+            self.addl_metadata = copyfrom.addl_metadata if copyfrom is not None else {}
+
+        if imports is not None:
+            self.imports = imports
+        else:
+            self.imports = copyfrom.imports if copyfrom is not None else []
+
+        if includes is not None:
+            self.includes = includes
+        else:
+            self.includes = copyfrom.includes if copyfrom is not None else []
+
+        if fetcher is not None:
+            self.fetcher = fetcher
+        elif copyfrom is not None:
+            self.fetcher = copyfrom.fetcher
+        else:
             import requests
             from cachecontrol.caches import FileCache
             from cachecontrol.wrapper import CacheControl
@@ -71,8 +130,8 @@ class LoadingOptions:
                 cache=FileCache(root / ".cache" / "salad"),
             )
             self.fetcher: Fetcher = DefaultFetcher({}, session)
-        else:
-            self.fetcher = fetcher
+
+        self.cache = self.fetcher.cache if isinstance(self.fetcher, MemoryCachingFetcher) else {}
 
         self.vocab = _vocab
         self.rvocab = _rvocab
@@ -84,8 +143,45 @@ class LoadingOptions:
                 self.vocab[k] = v
                 self.rvocab[v] = k
 
+    @property
+    def graph(self) -> Graph:
+        """Generate a merged rdflib.Graph from all entries in self.schemas."""
+        graph = Graph()
+        if not self.schemas:
+            return graph
+        key = str(hash(tuple(self.schemas)))
+        if key in self.cache:
+            return cast(Graph, self.cache[key])
+        for schema in self.schemas:
+            fetchurl = (
+                self.fetcher.urljoin(self.fileuri, schema)
+                if self.fileuri is not None
+                else pathlib.Path(schema).resolve().as_uri()
+            )
+            if fetchurl not in self.cache or self.cache[fetchurl] is True:
+                _logger.debug("Getting external schema %s", fetchurl)
+                try:
+                    content = self.fetcher.fetch_text(fetchurl)
+                except Exception as e:
+                    _logger.warning("Could not load extension schema %s: %s", fetchurl, str(e))
+                    continue
+                newGraph = Graph()
+                err_msg = "unknown error"
+                for fmt in ["xml", "turtle"]:
+                    try:
+                        newGraph.parse(data=content, format=fmt, publicID=str(fetchurl))
+                        self.cache[fetchurl] = newGraph
+                        graph += newGraph
+                        break
+                    except (xml.sax.SAXParseException, TypeError, BadSyntax) as e:
+                        err_msg = str(e)
+                else:
+                    _logger.warning("Could not load extension schema %s: %s", fetchurl, err_msg)
+        self.cache[key] = graph
+        return graph
 
-class Savable(ABC):
+
+class Saveable(ABC):
     """Mark classes than have a save() and fromDoc() function."""
 
     @classmethod
@@ -96,16 +192,14 @@ class Savable(ABC):
         baseuri: str,
         loadingOptions: LoadingOptions,
         docRoot: Optional[str] = None,
-    ) -> "Savable":
+    ) -> "Saveable":
         """Construct this object from the result of yaml.load()."""
-        pass
 
     @abstractmethod
     def save(
         self, top: bool = False, base_url: str = "", relative_uris: bool = True
     ) -> Dict[str, Any]:
         """Convert this object to a JSON/YAML friendly dictionary."""
-        pass
 
 
 def load_field(val, fieldtype, baseuri, loadingOptions):
@@ -114,45 +208,72 @@ def load_field(val, fieldtype, baseuri, loadingOptions):
         if "$import" in val:
             if loadingOptions.fileuri is None:
                 raise SchemaSaladException("Cannot load $import without fileuri")
-            return _document_load_by_url(
+            url = loadingOptions.fetcher.urljoin(loadingOptions.fileuri, val["$import"])
+            result, metadata = _document_load_by_url(
                 fieldtype,
-                loadingOptions.fetcher.urljoin(loadingOptions.fileuri, val["$import"]),
+                url,
                 loadingOptions,
             )
+            loadingOptions.imports.append(url)
+            return result
         elif "$include" in val:
             if loadingOptions.fileuri is None:
                 raise SchemaSaladException("Cannot load $import without fileuri")
-            val = loadingOptions.fetcher.fetch_text(
-                loadingOptions.fetcher.urljoin(loadingOptions.fileuri, val["$include"])
-            )
+            url = loadingOptions.fetcher.urljoin(loadingOptions.fileuri, val["$include"])
+            val = loadingOptions.fetcher.fetch_text(url)
+            loadingOptions.includes.append(url)
     return fieldtype.load(val, baseuri, loadingOptions)
 
 
-save_type = Union[Dict[str, Any], List[Union[Dict[str, Any], List[Any], None]], None]
+save_type = Optional[Union[MutableMapping[str, Any], MutableSequence[Any], int, float, bool, str]]
 
 
 def save(
-    val: Optional[Union[Savable, MutableSequence[Savable]]],
+    val: Any,
     top: bool = True,
     base_url: str = "",
     relative_uris: bool = True,
 ) -> save_type:
-
-    if isinstance(val, Savable):
+    if isinstance(val, Saveable):
         return val.save(top=top, base_url=base_url, relative_uris=relative_uris)
     if isinstance(val, MutableSequence):
-        return [
-            save(v, top=False, base_url=base_url, relative_uris=relative_uris)
-            for v in val
-        ]
+        return [save(v, top=False, base_url=base_url, relative_uris=relative_uris) for v in val]
     if isinstance(val, MutableMapping):
         newdict = {}
         for key in val:
-            newdict[key] = save(
-                val[key], top=False, base_url=base_url, relative_uris=relative_uris
-            )
+            newdict[key] = save(val[key], top=False, base_url=base_url, relative_uris=relative_uris)
         return newdict
-    return val
+    if val is None or isinstance(val, (int, float, bool, str)):
+        return val
+    raise Exception("Not Saveable: %s" % type(val))
+
+
+def save_with_metadata(
+    val: Any,
+    valLoadingOpts: LoadingOptions,
+    top: bool = True,
+    base_url: str = "",
+    relative_uris: bool = True,
+) -> save_type:
+    """Save and set $namespaces, $schemas, $base and any other metadata fields at the top level."""
+    saved_val = save(val, top, base_url, relative_uris)
+    newdict: MutableMapping[str, Any] = {}
+    if isinstance(saved_val, MutableSequence):
+        newdict = {"$graph": saved_val}
+    elif isinstance(saved_val, MutableMapping):
+        newdict = saved_val
+
+    if valLoadingOpts.namespaces:
+        newdict["$namespaces"] = valLoadingOpts.namespaces
+    if valLoadingOpts.schemas:
+        newdict["$schemas"] = valLoadingOpts.schemas
+    if valLoadingOpts.baseuri:
+        newdict["$base"] = valLoadingOpts.baseuri
+    for k, v in valLoadingOpts.addl_metadata.items():
+        if k not in newdict:
+            newdict[k] = v
+
+    return newdict
 
 
 def expand_url(
@@ -178,7 +299,7 @@ def expand_url(
     split = urlsplit(url)
 
     if (
-        (bool(split.scheme) and split.scheme in ["http", "https", "file"])
+        (bool(split.scheme) and split.scheme in loadingOptions.fetcher.supported_schemes())
         or url.startswith("$(")
         or url.startswith("${")
     ):
@@ -218,7 +339,7 @@ def expand_url(
             if url in loadingOptions.rvocab:
                 return loadingOptions.rvocab[url]
         else:
-            raise ValidationException(f"Term '{url}' not in vocabulary")
+            raise ValidationException(f"Term {url!r} not in vocabulary")
 
     return url
 
@@ -264,14 +385,12 @@ class _ArrayLoader(_Loader):
     def load(self, doc, baseuri, loadingOptions, docRoot=None):
         # type: (Any, str, LoadingOptions, Optional[str]) -> Any
         if not isinstance(doc, MutableSequence):
-            raise ValidationException("Expected a list")
+            raise ValidationException(f"Expected a list, was {type(doc)}")
         r = []  # type: List[Any]
         errors = []  # type: List[SchemaSaladException]
         for i in range(0, len(doc)):
             try:
-                lf = load_field(
-                    doc[i], _UnionLoader((self, self.items)), baseuri, loadingOptions
-                )
+                lf = load_field(doc[i], _UnionLoader((self, self.items)), baseuri, loadingOptions)
                 if isinstance(lf, MutableSequence):
                     r.extend(lf)
                 else:
@@ -287,9 +406,9 @@ class _ArrayLoader(_Loader):
 
 
 class _EnumLoader(_Loader):
-    def __init__(self, symbols):
-        # type: (Sequence[str]) -> None
+    def __init__(self, symbols: Sequence[str], name: str) -> None:
         self.symbols = symbols
+        self.name = name
 
     def load(self, doc, baseuri, loadingOptions, docRoot=None):
         # type: (Any, str, LoadingOptions, Optional[str]) -> Any
@@ -298,6 +417,9 @@ class _EnumLoader(_Loader):
         else:
             raise ValidationException(f"Expected one of {self.symbols}")
 
+    def __repr__(self):  # type: () -> str
+        return self.name
+
 
 class _SecondaryDSLLoader(_Loader):
     def __init__(self, inner):
@@ -316,22 +438,21 @@ class _SecondaryDSLLoader(_Loader):
                         r.append({"pattern": d})
                 elif isinstance(d, dict):
                     new_dict: Dict[str, Any] = {}
-                    if "pattern" in d:
-                        new_dict["pattern"] = d.pop("pattern")
+                    dict_copy = copy.deepcopy(d)
+                    if "pattern" in dict_copy:
+                        new_dict["pattern"] = dict_copy.pop("pattern")
                     else:
                         raise ValidationException(
-                            "Missing pattern in secondaryFiles specification entry: {}".format(
-                                d
-                            )
+                            f"Missing pattern in secondaryFiles specification entry: {d}"
                         )
                     new_dict["required"] = (
-                        d.pop("required") if "required" in d else None
+                        dict_copy.pop("required") if "required" in dict_copy else None
                     )
 
-                    if len(d):
+                    if len(dict_copy):
                         raise ValidationException(
                             "Unallowed values in secondaryFiles specification entry: {}".format(
-                                d
+                                dict_copy
                             )
                         )
                     r.append(new_dict)
@@ -342,21 +463,18 @@ class _SecondaryDSLLoader(_Loader):
                     )
         elif isinstance(doc, MutableMapping):
             new_dict = {}
-            if "pattern" in doc:
-                new_dict["pattern"] = doc.pop("pattern")
+            doc_copy = copy.deepcopy(doc)
+            if "pattern" in doc_copy:
+                new_dict["pattern"] = doc_copy.pop("pattern")
             else:
                 raise ValidationException(
-                    "Missing pattern in secondaryFiles specification entry: {}".format(
-                        doc
-                    )
+                    f"Missing pattern in secondaryFiles specification entry: {doc}"
                 )
-            new_dict["required"] = doc.pop("required") if "required" in doc else None
+            new_dict["required"] = doc_copy.pop("required") if "required" in doc_copy else None
 
-            if len(doc):
+            if len(doc_copy):
                 raise ValidationException(
-                    "Unallowed values in secondaryFiles specification entry: {}".format(
-                        doc
-                    )
+                    f"Unallowed values in secondaryFiles specification entry: {doc_copy}"
                 )
             r.append(new_dict)
 
@@ -372,17 +490,17 @@ class _SecondaryDSLLoader(_Loader):
 
 class _RecordLoader(_Loader):
     def __init__(self, classtype):
-        # type: (Type[Savable]) -> None
+        # type: (Type[Saveable]) -> None
         self.classtype = classtype
 
     def load(self, doc, baseuri, loadingOptions, docRoot=None):
         # type: (Any, str, LoadingOptions, Optional[str]) -> Any
         if not isinstance(doc, MutableMapping):
-            raise ValidationException("Expected a dict")
+            raise ValidationException(f"Expected a dict, was {type(doc)}")
         return self.classtype.fromDoc(doc, baseuri, loadingOptions, docRoot=docRoot)
 
     def __repr__(self):  # type: () -> str
-        return str(self.classtype)
+        return str(self.classtype.__name__)
 
 
 class _ExpressionLoader(_Loader):
@@ -392,13 +510,12 @@ class _ExpressionLoader(_Loader):
     def load(self, doc, baseuri, loadingOptions, docRoot=None):
         # type: (Any, str, LoadingOptions, Optional[str]) -> Any
         if not isinstance(doc, str):
-            raise ValidationException("Expected a str")
+            raise ValidationException(f"Expected a str, was {type(doc)}")
         return doc
 
 
 class _UnionLoader(_Loader):
-    def __init__(self, alternates):
-        # type: (Sequence[_Loader]) -> None
+    def __init__(self, alternates: Sequence[_Loader]) -> None:
         self.alternates = alternates
 
     def load(self, doc, baseuri, loadingOptions, docRoot=None):
@@ -408,9 +525,7 @@ class _UnionLoader(_Loader):
             try:
                 return t.load(doc, baseuri, loadingOptions, docRoot=docRoot)
             except ValidationException as e:
-                errors.append(
-                    ValidationException(f"tried {t.__class__.__name__} but", None, [e])
-                )
+                errors.append(ValidationException(f"tried {t} but", None, [e]))
         raise ValidationException("", None, errors, "-")
 
     def __repr__(self):  # type: () -> str
@@ -475,9 +590,7 @@ class _TypeDSLLoader(_Loader):
         if m:
             group1 = m.group(1)
             assert group1 is not None  # nosec
-            first = expand_url(
-                group1, baseuri, loadingOptions, False, True, self.refScope
-            )
+            first = expand_url(group1, baseuri, loadingOptions, False, True, self.refScope)
             second = third = None
             if bool(m.group(2)):
                 second = {"type": "array", "items": first}
@@ -552,56 +665,99 @@ class _IdMapLoader(_Loader):
         return self.inner.load(doc, baseuri, loadingOptions)
 
 
-def _document_load(loader, doc, baseuri, loadingOptions):
-    # type: (_Loader, Any, str, LoadingOptions) -> Any
+def _document_load(
+    loader: _Loader,
+    doc: Union[str, MutableMapping[str, Any], MutableSequence[Any]],
+    baseuri: str,
+    loadingOptions: LoadingOptions,
+    addl_metadata_fields: Optional[MutableSequence[str]] = None,
+) -> Tuple[Any, LoadingOptions]:
     if isinstance(doc, str):
         return _document_load_by_url(
-            loader, loadingOptions.fetcher.urljoin(baseuri, doc), loadingOptions
+            loader,
+            loadingOptions.fetcher.urljoin(baseuri, doc),
+            loadingOptions,
+            addl_metadata_fields=addl_metadata_fields,
         )
 
     if isinstance(doc, MutableMapping):
-        if "$namespaces" in doc or "$schemas" in doc:
-            loadingOptions = LoadingOptions(
-                copyfrom=loadingOptions,
-                namespaces=doc.get("$namespaces", None),
-                schemas=doc.get("$schemas", None),
-            )
-            doc = {k: v for k, v in doc.items() if k not in ["$namespaces", "$schemas"]}
+        addl_metadata = {}
+        if addl_metadata_fields is not None:
+            for mf in addl_metadata_fields:
+                if mf in doc:
+                    addl_metadata[mf] = doc[mf]
 
+        docuri = baseuri
         if "$base" in doc:
             baseuri = doc["$base"]
 
+        loadingOptions = LoadingOptions(
+            copyfrom=loadingOptions,
+            namespaces=doc.get("$namespaces", None),
+            schemas=doc.get("$schemas", None),
+            baseuri=doc.get("$base", None),
+            addl_metadata=addl_metadata,
+        )
+
+        doc = {k: v for k, v in doc.items() if k not in ("$namespaces", "$schemas", "$base")}
+
         if "$graph" in doc:
-            return loader.load(doc["$graph"], baseuri, loadingOptions)
+            loadingOptions.idx[baseuri] = (
+                loader.load(doc["$graph"], baseuri, loadingOptions),
+                loadingOptions,
+            )
         else:
-            return loader.load(doc, baseuri, loadingOptions, docRoot=baseuri)
+            loadingOptions.idx[baseuri] = (
+                loader.load(doc, baseuri, loadingOptions, docRoot=baseuri),
+                loadingOptions,
+            )
+
+        if docuri != baseuri:
+            loadingOptions.idx[docuri] = loadingOptions.idx[baseuri]
+
+        return loadingOptions.idx[baseuri]
 
     if isinstance(doc, MutableSequence):
-        return loader.load(doc, baseuri, loadingOptions)
+        loadingOptions.idx[baseuri] = (
+            loader.load(doc, baseuri, loadingOptions),
+            loadingOptions,
+        )
+        return loadingOptions.idx[baseuri]
 
-    raise ValidationException("Oops, we shouldn't be here!")
+    raise ValidationException(
+        "Expected URI string, MutableMapping or MutableSequence, got %s" % type(doc)
+    )
 
 
-def _document_load_by_url(loader, url, loadingOptions):
-    # type: (_Loader, str, LoadingOptions) -> Any
+def _document_load_by_url(
+    loader: _Loader,
+    url: str,
+    loadingOptions: LoadingOptions,
+    addl_metadata_fields: Optional[MutableSequence[str]] = None,
+) -> Tuple[Any, LoadingOptions]:
     if url in loadingOptions.idx:
-        return _document_load(loader, loadingOptions.idx[url], url, loadingOptions)
+        return loadingOptions.idx[url]
 
-    text = loadingOptions.fetcher.fetch_text(url)
-    if isinstance(text, bytes):
-        textIO = StringIO(text.decode("utf-8"))
-    else:
-        textIO = StringIO(text)
-    textIO.name = str(url)
+    doc_url, frg = urldefrag(url)
+
+    text = loadingOptions.fetcher.fetch_text(doc_url)
+    textIO = StringIO(text)
+    textIO.name = str(doc_url)
     yaml = yaml_no_ts()
     result = yaml.load(textIO)
-    add_lc_filename(result, url)
+    add_lc_filename(result, doc_url)
 
-    loadingOptions.idx[url] = result
+    loadingOptions = LoadingOptions(copyfrom=loadingOptions, fileuri=doc_url)
 
-    loadingOptions = LoadingOptions(copyfrom=loadingOptions, fileuri=url)
+    _document_load(
+        loader,
+        result,
+        doc_url,
+        loadingOptions,
+        addl_metadata_fields=addl_metadata_fields,
+    )
 
-    return _document_load(loader, result, url, loadingOptions)
+    return loadingOptions.idx[url]
 
 
 def file_uri(path, split_frag=False):  # type: (str, bool) -> str
@@ -636,14 +792,11 @@ def save_relative_uri(
     relative_uris: bool,
 ) -> Any:
     """Convert any URI to a relative one, obeying the scoping rules."""
-    if not relative_uris or uri == base_url:
-        return uri
     if isinstance(uri, MutableSequence):
-        return [
-            save_relative_uri(u, base_url, scoped_id, ref_scope, relative_uris)
-            for u in uri
-        ]
+        return [save_relative_uri(u, base_url, scoped_id, ref_scope, relative_uris) for u in uri]
     elif isinstance(uri, str):
+        if not relative_uris or uri == base_url:
+            return uri
         urisplit = urlsplit(uri)
         basesplit = urlsplit(base_url)
         if urisplit.scheme == basesplit.scheme and urisplit.netloc == basesplit.netloc:
@@ -668,4 +821,16 @@ def save_relative_uri(
                 return urisplit.fragment
         return uri
     else:
-        return save(uri, top=False, base_url=base_url)
+        return save(uri, top=False, base_url=base_url, relative_uris=relative_uris)
+
+
+def shortname(inputid: str) -> str:
+    """
+    Compute the shortname of a fully qualified identifier.
+
+    See https://w3id.org/cwl/v1.2/SchemaSalad.html#Short_names.
+    """
+    parsed_id = urlparse(inputid)
+    if parsed_id.fragment:
+        return parsed_id.fragment.split("/")[-1]
+    return parsed_id.path.split("/")[-1]
