@@ -34,7 +34,7 @@ from schema_salad.utils import (
 )
 
 from . import _logger, jsonld_context, ref_resolver, validate
-from .avro.schema import Names, SchemaParseException, make_avsc_object
+from .avro.schema import Names, SchemaParseException, is_subtype, make_avsc_object
 from .exceptions import (
     ClassValidationException,
     SchemaSaladException,
@@ -81,9 +81,15 @@ SALAD_FILES = (
 
 saladp = "https://w3id.org/cwl/salad#"
 
+cached_metaschema: Optional[Tuple[Names, List[Dict[str, str]], Loader]] = None
+
 
 def get_metaschema() -> Tuple[Names, List[Dict[str, str]], Loader]:
     """Instantiate the metaschema."""
+    global cached_metaschema
+    if cached_metaschema is not None:
+        return cached_metaschema
+
     loader = ref_resolver.Loader(
         {
             "Any": saladp + "Any",
@@ -176,9 +182,7 @@ def get_metaschema() -> Tuple[Names, List[Dict[str, str]], Loader]:
 
     for salad in SALAD_FILES:
         with resource_stream("schema_salad", "metaschema/" + salad) as stream:
-            loader.cache["https://w3id.org/cwl/" + salad] = stream.read().decode(
-                "UTF-8"
-            )
+            loader.cache["https://w3id.org/cwl/" + salad] = stream.read().decode("UTF-8")
 
     with resource_stream("schema_salad", "metaschema/metaschema.yml") as stream:
         loader.cache["https://w3id.org/cwl/salad"] = stream.read().decode("UTF-8")
@@ -199,12 +203,11 @@ def get_metaschema() -> Tuple[Names, List[Dict[str, str]], Loader]:
         _logger.error("Metaschema error, avro was:\n%s", json_dumps(sch_obj, indent=4))
         raise
     validate_doc(sch_names, j2, loader, strict=True)
-    return (sch_names, j2, loader)
+    cached_metaschema = (sch_names, j2, loader)
+    return cached_metaschema
 
 
-def add_namespaces(
-    metadata: Mapping[str, Any], namespaces: MutableMapping[str, str]
-) -> None:
+def add_namespaces(metadata: Mapping[str, Any], namespaces: MutableMapping[str, str]) -> None:
     """Collect the provided namespaces, checking for conflicts."""
     for key, value in metadata.items():
         if key not in namespaces:
@@ -237,12 +240,18 @@ def load_schema(
     """
     Load a schema that can be used to validate documents using load_and_validate.
 
-    return: document_loader, avsc_names, schema_metadata, metaschema_loader
+    :returns: document_loader, avsc_names, schema_metadata, metaschema_loader
     """
-
     metaschema_names, _metaschema_doc, metaschema_loader = get_metaschema()
     if cache is not None:
-        metaschema_loader.cache.update(cache)
+        # we want to replace some items in the cache, so we need to
+        # make a new Loader with an empty index.
+        for k, v in metaschema_loader.cache.items():
+            if k not in cache:
+                cache[k] = v
+        metaschema_loader = Loader(
+            ctx=metaschema_loader.ctx, cache=cache, session=metaschema_loader.session
+        )
     schema_doc, schema_metadata = metaschema_loader.resolve_ref(schema_ref, "")
 
     if not isinstance(schema_doc, MutableSequence):
@@ -356,7 +365,7 @@ def validate_doc(
                 break
 
         if not success:
-            errors = []  # type: List[SchemaSaladException]
+            errors: List[SchemaSaladException] = []
             for root in roots:
                 if hasattr(root, "get_prop"):
                     name = root.get_prop("name")
@@ -392,18 +401,14 @@ def validate_doc(
             objerr = "Invalid"
             for ident in loader.identifiers:
                 if ident in item:
-                    objerr = "Object `{}` is not valid because".format(
-                        relname(item[ident])
-                    )
+                    objerr = f"Object `{relname(item[ident])}` is not valid because"
                     break
             anyerrors.append(ValidationException(objerr, sourceline, errors, "-"))
     if anyerrors:
         raise ValidationException("", None, anyerrors, "*")
 
 
-def get_anon_name(
-    rec: MutableMapping[str, Union[str, Dict[str, str], List[str]]]
-) -> str:
+def get_anon_name(rec: MutableMapping[str, Union[str, Dict[str, str], List[str]]]) -> str:
     """Calculate a reproducible name for anonymous types."""
     if "name" in rec:
         name = rec["name"]
@@ -414,18 +419,14 @@ def get_anon_name(
     if rec["type"] in ("enum", saladp + "enum"):
         for sym in rec["symbols"]:
             anon_name += sym
-        return (
-            "anon.enum_" + hashlib.sha1(anon_name.encode("UTF-8")).hexdigest()  # nosec
-        )
+        return "anon.enum_" + hashlib.sha1(anon_name.encode("UTF-8")).hexdigest()  # nosec
     if rec["type"] in ("record", saladp + "record"):
         for field in rec["fields"]:
             if isinstance(field, Mapping):
                 anon_name += field["name"]
             else:
                 raise ValidationException(
-                    "Expected entries in 'fields' to also be maps, was {}.".format(
-                        field
-                    )
+                    f"Expected entries in 'fields' to also be maps, was {field}."
                 )
         return "record_" + hashlib.sha1(anon_name.encode("UTF-8")).hexdigest()  # nosec
     if rec["type"] in ("array", saladp + "array"):
@@ -488,9 +489,7 @@ def replace_type(
             replace_with = spec[items]
 
         if replace_with:
-            return replace_type(
-                replace_with, spec, loader, found, find_embeds=find_embeds
-            )
+            return replace_type(replace_with, spec, loader, found, find_embeds=find_embeds)
         found.add(items)
     return items
 
@@ -520,11 +519,8 @@ def make_valid_avro(
     union: bool = False,
     fielddef: bool = False,
     vocab: Optional[Dict[str, str]] = None,
-) -> Union[
-    Avro, MutableMapping[str, str], str, List[Union[Any, MutableMapping[str, str], str]]
-]:
+) -> Union[Avro, MutableMapping[str, str], str, List[Union[Any, MutableMapping[str, str], str]]]:
     """Convert our schema to be more avro like."""
-
     if vocab is None:
         _, _, metaschema_loader = get_metaschema()
         vocab = metaschema_loader.vocab
@@ -566,16 +562,12 @@ def make_valid_avro(
         ret = []
         for i in items:
             ret.append(
-                make_valid_avro(
-                    i, alltypes, found, union=union, fielddef=fielddef, vocab=vocab
-                )
+                make_valid_avro(i, alltypes, found, union=union, fielddef=fielddef, vocab=vocab)
             )
         return ret
     if union and isinstance(items, str):
         if items in alltypes and validate.avro_type_name(items) not in found:
-            return make_valid_avro(
-                alltypes[items], alltypes, found, union=union, vocab=vocab
-            )
+            return make_valid_avro(alltypes[items], alltypes, found, union=union, vocab=vocab)
         if items in vocab:
             return validate.avro_type_name(vocab[items])
         else:
@@ -591,7 +583,6 @@ def deepcopy_strip(item: Any) -> Any:
     Intentionally do not copy attributes.  This is to discard CommentedMap and
     CommentedSeq metadata which is very expensive with regular copy.deepcopy.
     """
-
     if isinstance(item, MutableMapping):
         return {k: deepcopy_strip(v) for k, v in item.items()}
     if isinstance(item, MutableSequence):
@@ -599,13 +590,8 @@ def deepcopy_strip(item: Any) -> Any:
     return item
 
 
-def extend_and_specialize(
-    items: List[Dict[str, Any]], loader: Loader
-) -> List[Dict[str, Any]]:
-    """
-    Apply 'extend' and 'specialize' to fully materialize derived record types.
-    """
-
+def extend_and_specialize(items: List[Dict[str, Any]], loader: Loader) -> List[Dict[str, Any]]:
+    """Apply 'extend' and 'specialize' to fully materialize derived record types."""
     items2 = deepcopy_strip(items)
     types = {i["name"]: i for i in items2}  # type: Dict[str, Any]
     results = []
@@ -617,7 +603,7 @@ def extend_and_specialize(
                 for spec in aslist(stype["specialize"]):
                     specs[spec["specializeFrom"]] = spec["specializeTo"]
 
-            exfields = []  # type: List[str]
+            exfields = []  # type: List[Any]
             exsym = []  # type: List[str]
             for ex in aslist(stype["extends"]):
                 if ex not in types:
@@ -645,23 +631,57 @@ def extend_and_specialize(
 
             if stype["type"] == "record":
                 stype = copy.copy(stype)
-                exfields.extend(stype.get("fields", []))
-                stype["fields"] = exfields
+                combined_fields = []
+                fields = stype.get("fields", [])
+                # We use short names here so that if a type inherits a field
+                # (e.g. Child#id) from a parent (Parent#id) we avoid adding
+                # the same field twice (previously we had just
+                # ``exfields.extends(stype.fields)``).
+                sns_fields = {shortname(field["name"]): field for field in fields}
+                sns_exfields = {shortname(exfield["name"]): exfield for exfield in exfields}
+
+                # N.B.: This could be simpler. We could have a single loop
+                #       to create the list of fields. The reason for this more
+                #       convoluted solution is to make sure we keep the order
+                #       of ``exfields`` first, and then the type fields. That's
+                #       because we have unit tests that rely on the order that
+                #       fields are written. Codegen output changes as well.
+                #       We are relying on the insertion order preserving
+                #       property of python dicts (i.e. relyig on Py3.5+).
+
+                # First pass adding the exfields.
+                for sn_exfield, exfield in sns_exfields.items():
+                    field = sns_fields.get(sn_exfield, None)
+                    if field is None:
+                        field = exfield
+                    else:
+                        # make sure field name has not been used yet
+                        if not is_subtype(exfield["type"], field["type"]):
+                            raise SchemaParseException(
+                                f"Field name {field['name']} already in use with "
+                                "incompatible type. "
+                                f"{field['type']} vs {exfield['type']}."
+                            )
+                    combined_fields.append(field)
+                # Second pass, now add the ones that are specific to the subtype.
+                for field in sns_fields.values():
+                    if field not in combined_fields:
+                        combined_fields.append(field)
+
+                stype["fields"] = combined_fields
 
                 fieldnames = set()  # type: Set[str]
                 for field in stype["fields"]:
                     if field["name"] in fieldnames:
                         raise ValidationException(
-                            "Field name {} appears twice in {}".format(
-                                field["name"], stype["name"]
-                            )
+                            "Field name {} appears twice in {}".format(field["name"], stype["name"])
                         )
                     else:
                         fieldnames.add(field["name"])
             elif stype["type"] == "enum":
                 stype = copy.copy(stype)
                 exsym.extend(stype.get("symbols", []))
-                stype["symbol"] = exsym
+                stype["symbols"] = exsym
 
             types[stype["name"]] = stype
 
@@ -687,9 +707,7 @@ def extend_and_specialize(
 
     for result in results:
         if "fields" in result:
-            result["fields"] = replace_type(
-                result["fields"], extended_by, loader, set()
-            )
+            result["fields"] = replace_type(result["fields"], extended_by, loader, set())
 
     return results
 
@@ -699,7 +717,6 @@ def make_avro(
     loader: Loader,
     metaschema_vocab: Optional[Dict[str, str]] = None,
 ) -> List[Any]:
-
     j = extend_and_specialize(i, loader)
 
     name_dict = {}  # type: Dict[str, Dict[str, Any]]
@@ -739,7 +756,7 @@ def make_avro_schema_from_avro(avro: List[Union[Avro, Dict[str, str], str]]) ->
 
 
 def shortname(inputid: str) -> str:
-    """Returns the last segment of the provided fragment or path."""
+    """Return the last segment of the provided fragment or path."""
     parsed_id = urlparse(inputid)
     if parsed_id.fragment:
         return parsed_id.fragment.split("/")[-1]
@@ -758,10 +775,10 @@ def print_inheritance(doc: List[Dict[str, Any]], stream: IO[Any]) -> None:
                     "\\l* ".join(shortname(field["name"]) for field in fields)
                 )
             shape = "ellipse" if entry.get("abstract") else "box"
-            stream.write(f'"{name}" [shape={shape} label="{label}"];\n')
+            stream.write(f'"{name}" [shape={shape} label="{label}"];\n')  # noqa: B907
             if "extends" in entry:
                 for target in aslist(entry["extends"]):
-                    stream.write(f'"{shortname(target)}" -> "{name}";\n')
+                    stream.write(f'"{shortname(target)}" -> "{name}";\n')  # noqa: B907
     stream.write("}\n")
 
 
