@@ -4,7 +4,6 @@ from typing import (
     Any,
     AnyStr,
     Callable,
-    Dict,
     List,
     MutableMapping,
     MutableSequence,
@@ -78,17 +77,13 @@ def reflow(text: str, maxline: int, shift: Optional[str] = "") -> str:
             if sp == -1:
                 sp = len(text)
         if sp < len(text):
-            return "{}\n{}{}".format(
-                text[0:sp], shift, reflow(text[sp + 1 :], maxline, shift)
-            )
+            return f"{text[0:sp]}\n{shift}{reflow(text[sp + 1 :], maxline, shift)}"
     return text
 
 
 def indent(v: str, nolead: bool = False, shift: str = "  ", bullet: str = "  ") -> str:
     if nolead:
-        return v.splitlines()[0] + "\n".join(
-            [shift + line for line in v.splitlines()[1:]]
-        )
+        return v.splitlines()[0] + "\n".join([shift + line for line in v.splitlines()[1:]])
     else:
 
         def lineno(i: int, line: str) -> str:
@@ -113,7 +108,11 @@ def bullets(textlist: List[str], bul: str) -> str:
 
 
 def strip_duplicated_lineno(text: str) -> str:
-    """Same as `strip_dup_lineno` but without reflow"""
+    """
+    Strip duplicated line numbers.
+
+    Same as :py:meth:`strip_dup_lineno` but without reflow.
+    """
     pre = None  # type: Optional[str]
     msg = []
     for line in text.splitlines():
@@ -173,7 +172,7 @@ def strip_dup_lineno(text: str, maxline: Optional[int] = None) -> str:
 
 
 def cmap(
-    d: Union[int, float, str, Dict[str, Any], List[Any], None],
+    d: Union[int, float, str, MutableMapping[str, Any], MutableSequence[Any], None],
     lc: Optional[List[int]] = None,
     fn: Optional[str] = None,
 ) -> Union[int, float, str, CommentedMap, CommentedSeq, None]:
@@ -264,11 +263,7 @@ class SourceLine:
     def start(self) -> Optional[Tuple[int, int]]:
         if self.file() is None:
             return None
-        elif (
-            self.key is None
-            or self.item.lc.data is None
-            or self.key not in self.item.lc.data
-        ):
+        elif self.key is None or self.item.lc.data is None or self.key not in self.item.lc.data:
             return ((self.item.lc.line or 0) + 1, (self.item.lc.col or 0) + 1)
         else:
             return (
