@@ -163,9 +163,7 @@ def test_idmap2() -> None:
 def test_load_pt() -> None:
     path = get_data("tests/pt.yml")
     assert path
-    doc = cg_metaschema.load_document(
-        file_uri(path), "", cg_metaschema.LoadingOptions()
-    )
+    doc = cg_metaschema.load_document(file_uri(path), "", cg_metaschema.LoadingOptions())
     assert [
         "https://w3id.org/cwl/salad#null",
         "http://www.w3.org/2001/XMLSchema#boolean",
@@ -177,6 +175,16 @@ def test_load_pt() -> None:
     ] == doc.symbols
 
 
+def test_shortname() -> None:
+    """Test shortname() function."""
+    assert cg_metaschema.shortname("http://example.com/foo") == "foo"
+    assert cg_metaschema.shortname("http://example.com/#bar") == "bar"
+    assert cg_metaschema.shortname("http://example.com/foo/bar") == "bar"
+    assert cg_metaschema.shortname("http://example.com/foo#bar") == "bar"
+    assert cg_metaschema.shortname("http://example.com/#foo/bar") == "bar"
+    assert cg_metaschema.shortname("http://example.com/foo#bar/baz") == "baz"
+
+
 @pytest.fixture
 def metaschema_pre() -> Any:
     """Prep-parsed schema for testing."""
