@@ -1,10 +1,15 @@
+"""Tests of helpful error messages."""
+import re
+
 import pytest
 
 import schema_salad
 import schema_salad.main
 from schema_salad.avro.schema import Names
 from schema_salad.exceptions import ValidationException
+from schema_salad.ref_resolver import Loader, file_uri
 from schema_salad.schema import load_and_validate, load_schema
+from schema_salad.sourceline import cmap
 
 from .util import get_data
 
@@ -71,11 +76,13 @@ def test_error_message2() -> None:
     assert isinstance(avsc_names, Names)
 
     t = "test_schema/test2.cwl"
-    match = r"""
-^.+test2\.cwl:2:1: Field `class`\s+contains\s+undefined\s+reference to
-\s+`file://.+/schema_salad/tests/test_schema/xWorkflow`$"""[
-        1:
-    ]
+    match = (
+        r"""
+^.+test2\.cwl:2:1: Field """
+        r"""`class`\s+contains\s+undefined\s+reference\s+to\s+`file://.+/schema_salad/tests/test_schema/xWorkflow`$"""[
+            1:
+        ]
+    )
     path2 = get_data("tests/" + t)
     assert path2
     with pytest.raises(ValidationException, match=match):
@@ -97,9 +104,7 @@ def test_error_message3() -> None:
         1:
     ]
     with pytest.raises(ValidationException, match=match):
-        load_and_validate(
-            document_loader, avsc_names, str(get_data("tests/" + t)), True
-        )
+        load_and_validate(document_loader, avsc_names, str(get_data("tests/" + t)), True)
 
 
 def test_error_message4() -> None:
@@ -112,13 +117,11 @@ def test_error_message4() -> None:
     match = r"""
 ^.+test4\.cwl:5:1: checking field\s+`outputs`
 .+test4\.cwl:6:3:   checking object\s+`.+test4\.cwl#bar`
-\s+`type` field is\s+int,\s+expected\s+string,\s+list, or\s+a\s+dict.$"""[
+\s+'type'\s+field\s+is\s+int,\s+expected\s+string,\s+list,\s+or\s+a\s+dict.$"""[
         1:
     ]
     with pytest.raises(ValidationException, match=match):
-        load_and_validate(
-            document_loader, avsc_names, str(get_data("tests/" + t)), True
-        )
+        load_and_validate(document_loader, avsc_names, str(get_data("tests/" + t)), True)
 
 
 def test_error_message5() -> None:
@@ -138,9 +141,7 @@ def test_error_message5() -> None:
         1:
     ]
     with pytest.raises(ValidationException, match=match):
-        load_and_validate(
-            document_loader, avsc_names, str(get_data("tests/" + t)), True
-        )
+        load_and_validate(document_loader, avsc_names, str(get_data("tests/" + t)), True)
 
 
 def test_error_message7() -> None:
@@ -162,9 +163,7 @@ def test_error_message7() -> None:
         + r"""'label',\s+'doc',\s+'run',\s+'scatter',\s+'scatterMethod'$"""
     )
     with pytest.raises(ValidationException, match=match):
-        load_and_validate(
-            document_loader, avsc_names, str(get_data("tests/" + t)), True
-        )
+        load_and_validate(document_loader, avsc_names, str(get_data("tests/" + t)), True)
 
 
 def test_error_message8() -> None:
@@ -174,17 +173,17 @@ def test_error_message8() -> None:
     assert isinstance(avsc_names, Names)
 
     t = "test_schema/test8.cwl"
-    match = r"""
+    match = (
+        r"""
 ^.+test8\.cwl:7:1: checking field\s+`steps`
 .+test8\.cwl:8:3:   checking object\s+`.+test8\.cwl#step1`
-.+test8\.cwl:9:5:     Field\s+`scatterMethod`\s+contains\s+undefined\s+reference\s+to
-\s+`file:///.+/tests/test_schema/abc`$"""[
-        1:
-    ]
+.+test8\.cwl:9:5:     """
+        r"""Field\s+`scatterMethod`\s+contains\s+undefined\s+reference\s+to\s+`file:///.+/tests/test_schema/abc`$"""[
+            1:
+        ]
+    )
     with pytest.raises(ValidationException, match=match):
-        load_and_validate(
-            document_loader, avsc_names, str(get_data("tests/" + t)), True
-        )
+        load_and_validate(document_loader, avsc_names, str(get_data("tests/" + t)), True)
 
 
 def test_error_message9() -> None:
@@ -197,13 +196,11 @@ def test_error_message9() -> None:
     match = (
         r"""^.+test9\.cwl:7:1: checking field\s+`steps`
 .+test9\.cwl:8:3:   checking object\s+`.+test9\.cwl#step1`
-.+test9\.cwl:9:5:     `scatterMethod`\s+field\s+is\s+"""
+.+test9\.cwl:9:5:     'scatterMethod'\s+field\s+is\s+"""
         + r"""int,\s+expected\s+string,\s+list,\s+or a\s+dict.$"""
     )
     with pytest.raises(ValidationException, match=match):
-        load_and_validate(
-            document_loader, avsc_names, str(get_data("tests/" + t)), True
-        )
+        load_and_validate(document_loader, avsc_names, str(get_data("tests/" + t)), True)
 
 
 def test_error_message10() -> None:
@@ -214,7 +211,7 @@ def test_error_message10() -> None:
 
     t = "test_schema/test10.cwl"
     match = r"""
-^.+test10\.cwl:2:1: Object\s+`.+test10\.cwl`\s+is not valid because
+^.+test10\.cwl:2:1: Object\s+`.+test10\.cwl`\s+is\s+not\s+valid\s+because
 \s+tried `Workflow`\s+but
 .+test10\.cwl:7:1:     the `steps`\s+field\s+is\s+not\s+valid\s+because
 \s+tried array\s+of\s+<WorkflowStep>\s+but
@@ -225,9 +222,7 @@ def test_error_message10() -> None:
         1:
     ]
     with pytest.raises(ValidationException, match=match):
-        load_and_validate(
-            document_loader, avsc_names, str(get_data("tests/" + t)), True
-        )
+        load_and_validate(document_loader, avsc_names, str(get_data("tests/" + t)), True)
 
 
 def test_error_message11() -> None:
@@ -237,17 +232,17 @@ def test_error_message11() -> None:
     assert isinstance(avsc_names, Names)
 
     t = "test_schema/test11.cwl"
-    match = r"""
+    match = (
+        r"""
 ^.+test11\.cwl:7:1: checking field\s+`steps`
 .+test11\.cwl:8:3:   checking object\s+`.+test11\.cwl#step1`
-.+test11\.cwl:9:5:     Field `run`\s+contains\s+undefined\s+reference to
-\s+`file://.+/tests/test_schema/blub\.cwl`$"""[
-        1:
-    ]
+.+test11\.cwl:9:5:     """
+        r"""Field `run`\s+contains\s+undefined\s+reference\s+to\s+`file://.+/tests/test_schema/blub\.cwl`$"""[
+            1:
+        ]
+    )
     with pytest.raises(ValidationException, match=match):
-        load_and_validate(
-            document_loader, avsc_names, str(get_data("tests/" + t)), True
-        )
+        load_and_validate(document_loader, avsc_names, str(get_data("tests/" + t)), True)
 
 
 def test_error_message15() -> None:
@@ -272,9 +267,7 @@ def test_error_message15() -> None:
         + r"""\s+'separate',\s+'itemSeparator',\s+'valueFrom',\s+'shellQuote'$"""
     )
     with pytest.raises(ValidationException, match=match):
-        load_and_validate(
-            document_loader, avsc_names, str(get_data("tests/" + t)), True
-        )
+        load_and_validate(document_loader, avsc_names, str(get_data("tests/" + t)), True)
 
 
 @pytest.mark.skip(
@@ -317,3 +310,139 @@ def test_bad_schema2() -> None:
     path = get_data("tests/bad_schema2.yml")
     assert path
     assert 1 == schema_salad.main.main(argsl=[path])
+
+
+def test_namespaces_type() -> None:
+    """Confirm helpful error message when $namespaces is the wrong type."""
+    with pytest.raises(
+        ValidationException,
+        match=re.escape("test:1:1: $namespaces must be a dictionary"),
+    ):
+        ldr, _, _, _ = schema_salad.schema.load_schema(
+            cmap(
+                {
+                    "$base": "Y",
+                    "name": "X",
+                    "$namespaces": ["http://example.com/foo#"],
+                    "$graph": [
+                        {
+                            "name": "ExampleType",
+                            "type": "enum",
+                            "symbols": ["asym", "bsym"],
+                        }
+                    ],
+                }
+            )
+        )
+
+
+def test_namespaces_undeclared(caplog: pytest.LogCaptureFixture) -> None:
+    """Confirm warning message a namespace is used but not declared."""
+
+    ldr, _, _, _ = schema_salad.schema.load_schema(
+        cmap(
+            {
+                "$base": "Y",
+                "name": "X",
+                "$graph": [
+                    {
+                        "name": "namesp:ExampleType",
+                        "type": "enum",
+                        "symbols": ["asym", "bsym"],
+                    }
+                ],
+            }
+        )
+    )
+
+    match = (
+        r""".*URI prefix 'namesp' of 'namesp:ExampleType' not recognized, """
+        r"""are you missing a \$namespaces section?.*"""
+    )
+
+    assert re.match(match, caplog.text)
+
+
+def test_not_a_namespace1(caplog: pytest.LogCaptureFixture) -> None:
+    """Confirm no warning when relative id contains a colon but prefix doesn't look like a namespace."""
+
+    ldr, _, _, _ = schema_salad.schema.load_schema(
+        cmap(
+            {
+                "$base": "Y",
+                "name": "X",
+                "$graph": [
+                    {
+                        "name": "foo/colon:ExampleType",
+                        "type": "enum",
+                        "symbols": ["asym", "bsym"],
+                    }
+                ],
+            }
+        )
+    )
+
+    assert caplog.text == ""
+
+
+def test_not_a_namespace2(caplog: pytest.LogCaptureFixture) -> None:
+    """Confirm no warning when relative id contains a colon but prefix doesn't look like a namespace."""
+
+    ldr, _, _, _ = schema_salad.schema.load_schema(
+        cmap(
+            {
+                "$base": "Y",
+                "name": "X",
+                "$graph": [
+                    {
+                        "name": "foo#colon:ExampleType",
+                        "type": "enum",
+                        "symbols": ["asym", "bsym"],
+                    }
+                ],
+            }
+        )
+    )
+
+    assert caplog.text == ""
+
+
+def test_not_a_namespace3(caplog: pytest.LogCaptureFixture) -> None:
+    """Confirm no warning when relative id starts with a colon."""
+
+    ldr, _, _, _ = schema_salad.schema.load_schema(
+        cmap(
+            {
+                "$base": "Y",
+                "name": "X",
+                "$graph": [
+                    {
+                        "name": ":ExampleType",
+                        "type": "enum",
+                        "symbols": ["asym", "bsym"],
+                    }
+                ],
+            }
+        )
+    )
+
+    assert caplog.text == ""
+
+
+def test_schemas_type() -> None:
+    """Confirm helpful error message when $schemas is the wrong type."""
+    path = get_data("tests/EDAM.owl")
+    assert path
+    with pytest.raises(
+        ValidationException,
+        match=re.escape("test:1:1: $schemas must be a string or a list of string"),
+    ):
+        ra, _ = Loader({}).resolve_all(
+            cmap(
+                {
+                    "$schemas": {"wrong": file_uri(path)},
+                    "edam:has_format": "edam:format_1915",
+                }
+            ),
+            "",
+        )
