@@ -4,7 +4,7 @@ import os
 from io import StringIO
 from typing import Any, Dict, cast
 
-from pytest import CaptureFixture
+import pytest
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
 import schema_salad.main
@@ -40,6 +40,51 @@ def test_schemas() -> None:
     } == ra
 
 
+def test_bad_schemas(caplog: pytest.LogCaptureFixture) -> None:
+    """Test that bad $schemas refs don't stop parsing."""
+    schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
+    document_path = get_data("tests/revtool_bad_schema.cwl")
+    assert schema_path and document_path
+    assert 0 == schema_salad.main.main(
+        argsl=["--print-rdf", schema_path, document_path],
+    )
+    assert (
+        "Could not load extension schema https://bad.example.com/missing.ttl: "
+        "Error fetching https://bad.example.com/missing.ttl"
+    ) in caplog.text
+    assert (
+        "https://schema.org/!DOCTYPE html does not look like a valid URI, "
+        "trying to serialize this will break"
+    ) in caplog.text
+    assert (
+        "Could not load extension schema https://schema.org/docs/schema_org_rdfa.html: "
+        "No plugin registered for (rdfa, <class 'rdflib.parser.Parser'>)"
+    ) in caplog.text
+
+
+def test_skip_bad_schemas(caplog: pytest.LogCaptureFixture) -> None:
+    """Test that (bad) $schemas refs are properly skipped."""
+    schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
+    document_path = get_data("tests/revtool_bad_schema.cwl")
+    assert schema_path and document_path
+    assert 0 == schema_salad.main.main(
+        argsl=["--print-rdf", "--skip-schemas", schema_path, document_path],
+    )
+    assert (
+        "Could not load extension schema https://bad.example.com/missing.ttl: "
+        "Error fetching https://bad.example.com/missing.ttl"
+    ) not in caplog.text
+    assert (
+        "https://schema.org/!DOCTYPE html does not look like a valid URI, "
+        "trying to serialize this will break"
+    ) not in caplog.text
+    assert (
+        "Could not load extension schema https://schema.org/docs/schema_org_rdfa.html: "
+        "No plugin registered for (rdfa, <class 'rdflib.parser.Parser'>)"
+    ) not in caplog.text
+    assert "rdflib.plugin.PluginException: No plugin registered" not in caplog.text
+
+
 def test_self_validate() -> None:
     path = get_data("metaschema/metaschema.yml")
     assert path
@@ -47,79 +92,67 @@ def test_self_validate() -> None:
     assert 0 == schema_salad.main.main(argsl=[path, path])
 
 
-def test_print_rdf(capfdbinary: CaptureFixture[bytes]) -> None:
+def test_print_rdf() -> None:
     """Test --print-rdf."""
     schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     document_path = get_data("tests/test_real_cwl/bio-cwl-tools/bamtools_stats.cwl")
     assert schema_path and document_path
-    assert 0 == schema_salad.main.main(
-        argsl=["--print-rdf", schema_path, document_path]
+    assert 0 == schema_salad.main.main(argsl=["--print-rdf", schema_path, document_path])
+
+
+def test_print_rdf_invalid_external_ref() -> None:
+    """Test --print-rdf when document references unfetchable external schema."""
+    schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
+    document_path = get_data(
+        "tests/test_real_cwl/bio-cwl-tools/bamtools_stats_invalid_schema_ref.cwl"
     )
-    captured = capfdbinary.readouterr()
-    assert captured.out
+    assert schema_path and document_path
+    assert 0 == schema_salad.main.main(argsl=["--print-rdf", schema_path, document_path])
 
 
-def test_print_pre_schema(capfdbinary: CaptureFixture[bytes]) -> None:
+def test_print_pre_schema() -> None:
     """Test --print-pre only schema."""
     schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     assert schema_path
     assert 0 == schema_salad.main.main(argsl=["--print-pre", schema_path])
-    captured = capfdbinary.readouterr()
-    assert captured.out
 
 
-def test_print_pre(capfdbinary: CaptureFixture[bytes]) -> None:
+def test_print_pre() -> None:
     """Test --print-pre."""
     schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     document_path = get_data("tests/test_real_cwl/bio-cwl-tools/bamtools_stats.cwl")
     assert schema_path and document_path
-    assert 0 == schema_salad.main.main(
-        argsl=["--print-pre", schema_path, document_path]
-    )
-    captured = capfdbinary.readouterr()
-    assert captured.out
+    assert 0 == schema_salad.main.main(argsl=["--print-pre", schema_path, document_path])
 
 
-def test_print_schema_index(capfdbinary: CaptureFixture[bytes]) -> None:
+def test_print_schema_index() -> None:
     """Test --print-index only with a schema."""
     schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     assert schema_path
     assert 0 == schema_salad.main.main(argsl=["--print-index", schema_path])
-    captured = capfdbinary.readouterr()
-    assert captured.out
 
 
-def test_print_index(capfdbinary: CaptureFixture[bytes]) -> None:
+def test_print_index() -> None:
     """Test --print-index."""
     schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     document_path = get_data("tests/test_real_cwl/bio-cwl-tools/bamtools_stats.cwl")
     assert schema_path and document_path
-    assert 0 == schema_salad.main.main(
-        argsl=["--print-index", schema_path, document_path]
-    )
-    captured = capfdbinary.readouterr()
-    assert captured.out
+    assert 0 == schema_salad.main.main(argsl=["--print-index", schema_path, document_path])
 
 
-def test_print_schema_metadata(capfdbinary: CaptureFixture[bytes]) -> None:
+def test_print_schema_metadata() -> None:
     """Test --print-metadata only for a schema."""
     schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     assert schema_path
     assert 0 == schema_salad.main.main(argsl=["--print-metadata", schema_path])
-    captured = capfdbinary.readouterr()
-    assert captured.out
 
 
-def test_print_metadata(capfdbinary: CaptureFixture[bytes]) -> None:
+def test_print_metadata() -> None:
     """Test --print-metadata."""
     schema_path = get_data("tests/test_schema/CommonWorkflowLanguage.yml")
     document_path = get_data("tests/test_real_cwl/bio-cwl-tools/bamtools_stats.cwl")
     assert schema_path and document_path
-    assert 0 == schema_salad.main.main(
-        argsl=["--print-metadata", schema_path, document_path]
-    )
-    captured = capfdbinary.readouterr()
-    assert captured.out
+    assert 0 == schema_salad.main.main(argsl=["--print-metadata", schema_path, document_path])
 
 
 def test_schema_salad_doc_oneline_doc() -> None:
@@ -144,9 +177,7 @@ def test_jsonld_ctx() -> None:
                 "$base": "Y",
                 "name": "X",
                 "$namespaces": {"foo": "http://example.com/foo#"},
-                "$graph": [
-                    {"name": "ExampleType", "type": "enum", "symbols": ["asym", "bsym"]}
-                ],
+                "$graph": [{"name": "ExampleType", "type": "enum", "symbols": ["asym", "bsym"]}],
             }
         )
     )
@@ -352,9 +383,7 @@ def test_scoped_id() -> None:
     }  # type: ContextType
     ldr.add_context(ctx)
 
-    ra, _ = ldr.resolve_all(
-        cmap({"id": "foo", "bar": {"id": "baz"}}), "http://example.com"
-    )
+    ra, _ = ldr.resolve_all(cmap({"id": "foo", "bar": {"id": "baz"}}), "http://example.com")
     assert {
         "id": "http://example.com/#foo",
         "bar": {"id": "http://example.com/#foo/baz"},
@@ -457,9 +486,7 @@ def test_subscoped_id() -> None:
     }  # type: ContextType
     ldr.add_context(ctx)
 
-    ra, _ = ldr.resolve_all(
-        cmap({"id": "foo", "bar": {"id": "baz"}}), "http://example.com"
-    )
+    ra, _ = ldr.resolve_all(cmap({"id": "foo", "bar": {"id": "baz"}}), "http://example.com")
     assert {
         "id": "http://example.com/#foo",
         "bar": {"id": "http://example.com/#foo/bar/baz"},
@@ -498,12 +525,8 @@ def test_file_uri() -> None:
     # Note: this test probably won't pass on Windows.  Someone with a
     # windows box should add an alternate test.
     assert "file:///foo/bar%20baz/quux" == file_uri("/foo/bar baz/quux")
-    assert os.path.normpath("/foo/bar baz/quux") == uri_file_path(
-        "file:///foo/bar%20baz/quux"
-    )
-    assert "file:///foo/bar%20baz/quux%23zing%20zong" == file_uri(
-        "/foo/bar baz/quux#zing zong"
-    )
+    assert os.path.normpath("/foo/bar baz/quux") == uri_file_path("file:///foo/bar%20baz/quux")
+    assert "file:///foo/bar%20baz/quux%23zing%20zong" == file_uri("/foo/bar baz/quux#zing zong")
     assert "file:///foo/bar%20baz/quux#zing%20zong" == file_uri(
         "/foo/bar baz/quux#zing zong", split_frag=True
     )
@@ -527,7 +550,7 @@ def test_sourceline() -> None:
     except TestExp as e:
         assert str(e).endswith("frag.yml:3:3: Whoops"), e
     except Exception as exc:
-        assert False, exc
+        raise AssertionError("Unexpected exception" + str(exc)) from exc
 
 
 def test_cmap() -> None:
