@@ -150,9 +150,7 @@ def test_for_invalid_yaml1() -> None:
             msg = str(e)
             print("\n", e)
             assert re.search(src + r":10:7: while scanning a\s+simple\s+key", msg, re.M)
-            assert re.search(
-                src + r":11:1:   could not\s+find\s+expected ':'$", msg, re.M
-            )
+            assert re.search(src + r":11:1:   could not\s+find\s+expected ':'$", msg, re.M)
             raise
 
 
