@@ -4,9 +4,9 @@ Checks loading of some real world tools and workflows found in the wild (e.g. do
 run individually as py.test -k tests/test_real_cwl.py
 """
 
-from typing import Any, Dict, Union
+from typing import Any, Dict, Optional, Union
 
-import pytest  # type: ignore
+import pytest
 
 from schema_salad.avro.schema import Names, SchemaParseException
 from schema_salad.exceptions import ValidationException
@@ -19,11 +19,10 @@ test_dir_name = "tests/test_real_cwl/"
 
 
 class TestRealWorldCWL:
-
-    document_loader = None  # type: Loader
-    avsc_names = None  # type: Union[Names, SchemaParseException]
-    schema_metadata = None  # type: Dict[str, Any]
-    metaschema_loader = None  # type: Loader
+    document_loader: Loader
+    avsc_names: Union[Names, SchemaParseException, None] = None
+    schema_metadata: Optional[Dict[str, Any]] = None
+    metaschema_loader: Optional[Loader] = None
 
     @classmethod
     def setup_class(cls) -> None:
@@ -43,7 +42,10 @@ class TestRealWorldCWL:
         with pytest.raises(ValidationException):
             try:
                 load_and_validate(
-                    self.document_loader, self.avsc_names, path, True,
+                    self.document_loader,
+                    self.avsc_names,
+                    path,
+                    True,
                 )
             except ValidationException as e:
                 # msgs = to_one_line_messages(str(e)).splitlines()
