@@ -11,6 +11,7 @@ from typing import (
     Iterable,
     Mapping,
     MutableSequence,
+    Optional,
     Tuple,
     TypeVar,
     Union,
@@ -29,7 +30,8 @@ ContextType = Dict[str, Union[Dict[str, Any], str, Iterable[str]]]
 DocumentType = TypeVar("DocumentType", CommentedSeq, CommentedMap)
 DocumentOrStrType = TypeVar("DocumentOrStrType", CommentedSeq, CommentedMap, str)
 FieldType = TypeVar("FieldType", str, CommentedSeq, CommentedMap)
-ResolveType = Union[int, float, str, CommentedMap, CommentedSeq, None]
+MandatoryResolveType = Union[int, float, str, CommentedMap, CommentedSeq]
+ResolveType = Optional[MandatoryResolveType]
 ResolvedRefType = Tuple[ResolveType, CommentedMap]
 IdxResultType = Union[CommentedMap, CommentedSeq, str, None]
 IdxType = Dict[str, IdxResultType]
@@ -44,24 +46,21 @@ def add_dictlist(di, key, val):  # type: (Dict[Any, Any], Any, Any) -> None
     di[key].append(val)
 
 
-def aslist(thing):  # type: (Any) -> MutableSequence[Any]
+def aslist(thing: Any) -> MutableSequence[Any]:
     """
-    Convenience function to wrap single items and lists.
+    Wrap single items and lists.
 
     Return lists unchanged.
     """
-
     if isinstance(thing, MutableSequence):
         return thing
     else:
         return [thing]
 
 
-# http://rightfootin.blogspot.com/2006/09/more-on-python-flatten.html
-
-
 def flatten(thing, ltypes=(list, tuple)):
     # type: (Any, Any) -> Any
+    # http://rightfootin.blogspot.com/2006/09/more-on-python-flatten.html
     if thing is None:
         return []
     if not isinstance(thing, ltypes):
@@ -97,18 +96,14 @@ def convert_to_dict(j4):  # type: (Any) -> Any
         return j4
 
 
-def json_dump(
-    obj,  # type: Any
-    fp,  # type: IO[str]
-    **kwargs  # type: Any
-):  # type: (...) -> None
+def json_dump(obj: Any, fp: IO[str], **kwargs: Any) -> None:
     """Force use of unicode."""
     json.dump(convert_to_dict(obj), fp, **kwargs)
 
 
 def json_dumps(
     obj,  # type: Any
-    **kwargs  # type: Any
+    **kwargs,  # type: Any
 ):  # type: (...) -> str
     """Force use of unicode."""
     return json.dumps(convert_to_dict(obj), **kwargs)
